﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml.XPath;
using System.Xml;
using System.Collections.Specialized;
using System.Threading;

namespace ForceSkypeHDVideo
{
    
    public partial class FormMain : Form
    {
        private XmlController xmlCtrl = new XmlController();

        public FormMain()
        {
            InitializeComponent();

            loadConfig();
        }

        private XmlNode getVideoXmlNode(string fullnameConfigXml)
        {
            XPathNavigator xPathNavigator = null;

            try
            {
                // try to open fullnameConfigXml StreamReader
                xPathNavigator = xmlCtrl.XPathNavigatorLoadXml(fullnameConfigXml);
            }
            catch (Exception ex)
            {
                MessageBox.Show("ERROR:\r\nCouldn't load configuration file " + fullnameConfigXml + "\r\n\r\nException: " + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                xPathNavigator = null;
            }

            if (xPathNavigator != null)
            {
                // Select variantConfiguration XmlNode
                XPathNodeIterator nodeIterator = xPathNavigator.Select(Properties.Settings.Default.xPathVideo);
                if (nodeIterator.Count <= 0) {
                    MessageBox.Show("ERROR:\r\nCouldn't find " + Properties.Settings.Default.xPathVideo + " in configuration file " + fullnameConfigXml, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return null;
                }
                else if (nodeIterator.Count > 1)
                {
                    MessageBox.Show("ERROR:\r\nFound more than one " + Properties.Settings.Default.xPathVideo + " in configuration file " + fullnameConfigXml, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return null;
                }
                else
                {
                    // MessageBox.Show("DEBUG:\r\nFound one " + Properties.Settings.Default.xPathVideo + " in configuration file " + fullnameConfigXml, "DEBUG", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    // get first node
                    nodeIterator.MoveNext();
                    return ((IHasXmlNode)nodeIterator.Current).GetNode();
                }
            }
            else
            {
                return null;
            }
        }

        private void buttonEnableHqVideo_Click(object sender, EventArgs e)
        {
            EnableVideoSetting(640, 480, 30, Properties.Settings.Default.PathSkypeConfig);
        }

        private void buttonEnable800x600Video_Click(object sender, EventArgs e)
        {
            EnableVideoSetting(800, 600, 30, Properties.Settings.Default.PathSkypeConfig);
        }

        private void buttonEnableHDTVVideo_Click(object sender, EventArgs e)
        {
            EnableVideoSetting(980, 720, 30, Properties.Settings.Default.PathSkypeConfig);
        }

        private void buttonEnableCustomVideo_Click(object sender, EventArgs e)
        {
            EnableVideoSetting(
                (int)Properties.Settings.Default.CaptureWidth,
                (int)Properties.Settings.Default.CaptureHeight,
                (int)Properties.Settings.Default.CaptureFps,
                Properties.Settings.Default.PathSkypeConfig
                );
        }

        private void EnableVideoSetting(int width, int height, int fps, string skypeConfigPath)
        {
            foreach (int i in checkedListBoxAccounts.CheckedIndices)
            {
                string fullnameConfigXml = skypeConfigPath.ToLower().Replace("%appdata%", System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)) + '\\' + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]) + '\\' + Properties.Settings.Default.FilenameConfig;
                XmlNode node = getVideoXmlNode(fullnameConfigXml);

                if (node != null)
                {
                    // Remove existing elements to avoid duplication
                    foreach (XmlNode nodeToRemove in node.SelectNodes("./" + Properties.Settings.Default.elementVideoWidth + " | ./" + Properties.Settings.Default.elementVideoHeight + " | ./" + Properties.Settings.Default.elementVideoFps))
                    {
                        node.RemoveChild(nodeToRemove);
                    }

                    // CaptureHeight element
                    XmlElement elemCaptureHeight = xmlCtrl.xmlDocument.CreateElement(Properties.Settings.Default.elementVideoHeight);
                    elemCaptureHeight.InnerText = height.ToString();
                    node.AppendChild(elemCaptureHeight);

                    // CaptureWidth element
                    XmlElement elemCaptureWidth = xmlCtrl.xmlDocument.CreateElement(Properties.Settings.Default.elementVideoWidth);
                    elemCaptureWidth.InnerText = width.ToString();
                    node.AppendChild(elemCaptureWidth);

                    // Fps element
                    XmlElement elemCaptureFps = xmlCtrl.xmlDocument.CreateElement(Properties.Settings.Default.elementVideoFps);
                    elemCaptureFps.InnerText = fps.ToString();
                    node.AppendChild(elemCaptureFps);

                    // Save the xml file
                    if (xmlCtrl.Save(fullnameConfigXml))
                    {
                        MessageBox.Show("Enabled " + width.ToString() + "*" + height.ToString() + " Video setting for " + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]), "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        MessageBox.Show("Error while enabling Video setting for " + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]) + "\r\n\r\nCouldn't save " + fullnameConfigXml, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show("Error while enabling Video setting for " + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]) + ", nothing was changed/written", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

            }
        }

        private void buttonDisableHqVideo_Click(object sender, EventArgs e)
        {
            foreach (int i in checkedListBoxAccounts.CheckedIndices)
            {
                string fullnameConfigXml = Properties.Settings.Default.PathSkypeConfig.ToLower().Replace("%appdata%", System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)) + '\\' + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]) + '\\' + Properties.Settings.Default.FilenameConfig;
                XmlNode node = getVideoXmlNode(fullnameConfigXml);

                if (node != null)
                {
                    // Remove HQ Video settings elements
                    foreach (XmlNode nodeToRemove in node.SelectNodes("./" + Properties.Settings.Default.elementVideoWidth + " | ./" + Properties.Settings.Default.elementVideoHeight + " | ./" + Properties.Settings.Default.elementVideoFps))
                    {
                        node.RemoveChild(nodeToRemove);
                    }

                    // Save the xml file
                    if (xmlCtrl.Save(fullnameConfigXml))
                    {
                        MessageBox.Show("Disabled Video setting for " + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]), "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        MessageBox.Show("Error while disabling Video setting for " + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]) + "\r\n\r\nCouldn't save " + fullnameConfigXml, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show("Error while disabling Video setting for " + checkedListBoxAccounts.GetItemText(checkedListBoxAccounts.Items[i]) + ", nothing was changed/written", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void loadConfig()
        {
            Properties.Settings.Default.Reload();

            /* Refresh account list */
            checkedListBoxAccounts.Items.Clear();
            try
            {
                DirectoryInfo dir = new DirectoryInfo(Properties.Settings.Default.PathSkypeConfig.ToLower().Replace("%appdata%", System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)));
                // MessageBox.Show("DEBUG:\r\nSkype config folder: "+dir.FullName,"DEBUG", MessageBoxButtons.OK, MessageBoxIcon.Information);
                if (dir.Exists)
                {
                    foreach (DirectoryInfo folder in dir.GetDirectories("*"))
                    {
                        FileInfo fileConfigXml = new FileInfo(folder.FullName + '\\' + Properties.Settings.Default.FilenameConfig);
                        // MessageBox.Show("DEBUG:\r\nTrying to find config file:" + fileConfigXml.FullName, "DEBUG", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        if (fileConfigXml.Exists)
                        {
                            int addedItemIndex = checkedListBoxAccounts.Items.Add(folder.Name);
                            checkedListBoxAccounts.SetItemChecked(addedItemIndex,true);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("ERROR:\r\nWhile trying to get account names !\r\nPath to Skype Config: " + Properties.Settings.Default.PathSkypeConfig + "\r\n\r\n" + "Exception message: " + e.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSaveConfig_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.Save();
            loadConfig();
        }

        private void buttonResetConfig_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.Reset();
            loadConfig();
        }

        private void linkLabelNodewave_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Help.ShowHelp(this, "http://www.nodewave.com");
        }

        private void linkLabelAmazon_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Help.ShowHelp(this, "http://www.amazon.com/?tag=nodewave-20&linkCode=ur2&camp=1789&creative=9325");
        }

    }

    public class XmlController
    {
        #region Enums

        #endregion

        #region Attributes
        public XmlDataDocument xmlDocument;

        #endregion

        #region Constructors
        /// <summary>
        /// Initialization.
        /// </summary>
        public XmlController()
        {
            // Private fields

            // Add general .Net objects

            // Add project specific objects

            // Execute optional functions needed for construction

        }
        #endregion

        #region Cleanup

        #endregion

        #region Methods & Properties

        private bool LoadXsdToDataset(String xsdFilename)
        {
            // Loads a specified schema into the DataSet
            StreamReader xsdStreamReader = null;
            try
            {
                xsdStreamReader = new StreamReader(xsdFilename);
                xmlDocument.DataSet.Reset();
                xmlDocument.DataSet.ReadXmlSchema(xsdStreamReader);
                Console.WriteLine("FileLoaded ", xsdFilename);
            }
            catch (Exception e)
            {
                Console.WriteLine("ERROR FileNotLoaded ", xsdFilename);
                Console.WriteLine("ERROR Exception ", e.ToString());
                return false;
            }
            finally
            {
                if (xsdStreamReader != null)
                    xsdStreamReader.Close();
            }
            return true;
        }

        private bool LoadXmlFile(String xmlFilename)
        {
            // Load the XML from file
            try
            {
                Console.WriteLine();

                xmlDocument.Load(xmlFilename);

                Console.WriteLine("FileLoaded ", xmlFilename);

            }
            catch (Exception e)
            {
                Console.WriteLine("ERROR FileNotLoaded ", xmlFilename);
                Console.WriteLine("ERROR Exception ", e.ToString());
                return false;
            }
            return true;
        }

        private void DisplayXmlDocument()
        {
            // Display the XML document.
            xmlDocument.Save(Console.Out);
            Console.WriteLine();
        }

        private void DisplayDatasetStructure()
        {
            // Displays the DataSet tables structure
            Console.WriteLine("Table Structure \r\n");
            Console.WriteLine("Number of tables=" + xmlDocument.DataSet.Tables.Count.ToString());
            for (int i = 0; i < xmlDocument.DataSet.Tables.Count; i++)
            {
                Console.WriteLine("\r\nTableName='" + xmlDocument.DataSet.Tables[i].TableName + "'.");
                Console.WriteLine("Number of columns=" + xmlDocument.DataSet.Tables[i].Columns.Count.ToString());

                for (int j = 0; j < xmlDocument.DataSet.Tables[i].Columns.Count; j++)
                {
                    Console.WriteLine("\tColumnName='" + xmlDocument.DataSet.Tables[i].Columns[j].ColumnName + "', type = " + xmlDocument.DataSet.Tables[i].Columns[j].DataType.ToString());

                }
            }
        }

        private void DisplayDataset()
        {
            // Displays the DataSet tables structure
            foreach (DataTable table in xmlDocument.DataSet.Tables)
            {
                Console.WriteLine("TableName = " + table.TableName);
                Console.WriteLine("{0}", "---------");
                Console.WriteLine("Columns ...");

                foreach (DataColumn column in table.Columns)
                {
                    Console.Write("{0,-22}", column.ColumnName);
                }
                Console.WriteLine("\r\n" + "Number of lines = {0}", table.Rows.Count.ToString());
                Console.WriteLine("Lines ...");

                foreach (DataRow row in table.Rows)
                {
                    foreach (Object value in row.ItemArray)
                    {
                        Console.Write("{0,-22}", value.ToString());
                    }
                    Console.WriteLine();
                }
                Console.WriteLine();
            }
        }

        public bool LoadDataset(String xmlFilename, String xsdFilename)
        {
            xmlDocument = new XmlDataDocument();

            bool XsdResult = LoadXsdToDataset(xsdFilename);

            //	DisplayDatasetStructure();

            bool XmlResult = LoadXmlFile(xmlFilename);

            //	DisplayXmlDocument();

            //	DisplayDataset();

            return XsdResult && XmlResult;
        }

        public bool Save(String xmlFile)
        {
            int retry = 5;
            bool result = false;
            Exception lastException = null;

            do
            {
                try
                {
                    xmlDocument.Save(xmlFile);
                    result = true;
                }
                catch (IOException e)
                {
                    // I don't understand why sometimes Save doesn't release stream on xml file, causing a IOException
                    lastException = e;
                    Thread.Sleep(500);
                    retry -= 1;
                }
            } while (retry > 0 && result == false);

            if (!result)
            {
                Console.WriteLine("ERROR FileNotSaved ", xmlFile);
                Console.WriteLine("ERROR Exception ", lastException.Message.ToString());
            }
            else
            {
                Console.WriteLine("FileSaved ", xmlFile);
            }


            return result;
        }

        public XmlNodeReader NodeReaderLoadXml(String xmlFilename)
        {
            xmlDocument = new XmlDataDocument();

            if (LoadXmlFile(xmlFilename))
            {
                //	DisplayXmlDocument();
                return new XmlNodeReader(xmlDocument);
            }
            else
            {
                return null;
            }
        }

        public XPathNavigator XPathNavigatorLoadXml(String xmlFilename)
        {
            if (NodeReaderLoadXml(xmlFilename) != null)
            {
                XPathNavigator xPathNavigator = xmlDocument.CreateNavigator();
                return xPathNavigator;
            }
            else
            {
                return null;
            }
        }

        public StringCollection LoadAndExtractColumn(String columnName, String xmlFilename, String xsdFilename)
        {
            if (LoadDataset(xmlFilename, xsdFilename))
            {
                // Displays the DataSet tables structure
                //			Console.WriteLine("Parse Dataset's Tables");
                foreach (DataTable table in xmlDocument.DataSet.Tables)
                {
                    int columnNumber = -1;

                    //				Console.WriteLine("Parse Table \"{0}\" Columns", table.ToString());
                    foreach (DataColumn column in table.Columns)
                    {
                        if (String.Compare(column.ColumnName, columnName) == 0)
                        {
                            columnNumber = column.Ordinal;
                            //						Console.WriteLine("Column \"{0}\" found.", columnName);
                        }
                    }

                    if (columnNumber < 0)
                    {
                        //					Console.WriteLine("Error: Column \"{0}\" was not found  !", columnName);
                    }
                    else
                    {
                        StringCollection Result = new StringCollection();
                        String resultValue;

                        foreach (DataRow row in table.Rows)
                        {
                            resultValue = row.ItemArray.GetValue(columnNumber).ToString();
                            Console.WriteLine(resultValue);
                            Result.Add(resultValue);
                        }

                        return Result;
                    }

                }
                return null;
            }

            return null;
        }

        #endregion

        #region EventHandlers

        #endregion
    }

}
